@@ -7,6 +7,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"net"
 	"net/url"
 	"os"
@@ -18,6 +19,8 @@ import (
 	"syscall"
 	"text/tabwriter"
 
+	"golang.org/x/crypto/ssh/terminal"
+
 	"github.com/influxdata/influxdb/client"
 	"github.com/influxdata/influxdb/importer/v8"
 	"github.com/influxdata/influxdb/models"
@@ -58,6 +61,7 @@ type CommandLine struct {
 	Chunked          bool
 	Quit             chan struct{}
 	IgnoreSignals    bool // Ignore signals normally caught by this process (used primarily for testing)
+	ForceTTY         bool // Force the CLI to act as if it were connected to a TTY
 	osSignals        chan os.Signal
 	historyFilePath  string
 }
@@ -73,10 +77,7 @@ func New(version string) *CommandLine {
 
 // Run executes the CLI
 func (c *CommandLine) Run() error {
-	if !c.IgnoreSignals {
-		// register OS signals for graceful termination
-		signal.Notify(c.osSignals, syscall.SIGINT, syscall.SIGTERM)
-	}
+	hasTTY := c.ForceTTY || terminal.IsTerminal(int(os.Stdin.Fd()))
 
 	var promptForPassword bool
 	// determine if they set the password flag but provided no value
@@ -88,18 +89,29 @@ func (c *CommandLine) Run() error {
 		}
 	}
 
-	c.Line = liner.NewLiner()
-	defer c.Line.Close()
-
-	c.Line.SetMultiLineMode(true)
+	// Check if we will be able to prompt for the password later.
+	if promptForPassword && !hasTTY {
+		return errors.New("Unable to prompt for a password with no TTY.")
+	}
 
+	// Read environment variables for username/password.
+	if c.Username == "" {
+		c.Username = os.Getenv("INFLUX_USERNAME")
+	}
+	// If we are going to be prompted for a password, always use the entered password.
 	if promptForPassword {
-		p, e := c.Line.PasswordPrompt("password: ")
+		// Open the liner (temporarily) and prompt for the password.
+		p, e := func() (string, error) {
+			l := liner.NewLiner()
+			defer l.Close()
+			return l.PasswordPrompt("password: ")
+		}()
 		if e != nil {
-			fmt.Println("Unable to parse password.")
-		} else {
-			c.Password = p
+			return errors.New("Unable to parse password")
 		}
+		c.Password = p
+	} else if c.Password == "" {
+		c.Password = os.Getenv("INFLUX_PASSWORD")
 	}
 
 	if err := c.Connect(""); err != nil {
@@ -111,17 +123,6 @@ func (c *CommandLine) Run() error {
 	// Modify precision.
 	c.SetPrecision(c.Precision)
 
-	if c.Execute == "" && !c.Import {
-		token, err := c.DatabaseToken()
-		if err != nil {
-			return fmt.Errorf("Failed to check token: %s", err.Error())
-		}
-		if token == "" {
-			fmt.Printf(noTokenMsg)
-		}
-		fmt.Printf("Connected to %s version %s\n", c.Client.Addr(), c.ServerVersion)
-	}
-
 	if c.Execute != "" {
 		// Make the non-interactive mode send everything through the CLI's parser
 		// the same way the interactive mode works
@@ -131,8 +132,6 @@ func (c *CommandLine) Run() error {
 				return err
 			}
 		}
-
-		c.Line.Close()
 		return nil
 	}
 
@@ -158,13 +157,38 @@ func (c *CommandLine) Run() error {
 		i := v8.NewImporter(config)
 		if err := i.Import(); err != nil {
 			err = fmt.Errorf("ERROR: %s\n", err)
-			c.Line.Close()
 			return err
 		}
-		c.Line.Close()
 		return nil
 	}
 
+	if !hasTTY {
+		cmd, err := ioutil.ReadAll(os.Stdin)
+		if err != nil {
+			return err
+		}
+		return c.ExecuteQuery(string(cmd))
+	}
+
+	if !c.IgnoreSignals {
+		// register OS signals for graceful termination
+		signal.Notify(c.osSignals, syscall.SIGINT, syscall.SIGTERM)
+	}
+
+	c.Line = liner.NewLiner()
+	defer c.Line.Close()
+
+	c.Line.SetMultiLineMode(true)
+
+	token, err := c.DatabaseToken()
+	if err != nil {
+		return fmt.Errorf("Failed to check token: %s", err.Error())
+	}
+	if token == "" {
+		fmt.Printf(noTokenMsg)
+	}
+	fmt.Printf("Connected to %s version %s\n", c.Client.Addr(), c.ServerVersion)
+
 	c.Version()
 
 	// Only load/write history if HOME environment variable is set.
@@ -200,7 +224,7 @@ func (c *CommandLine) mainLoop() error {
 				c.exit()
 				return e
 			}
-			if err := c.ParseCommand(l); err != ErrBlankCommand {
+			if err := c.ParseCommand(l); err != ErrBlankCommand && !strings.HasPrefix(strings.TrimSpace(l), "auth") {
 				c.Line.AppendHistory(l)
 				c.saveHistory()
 			}
@@ -570,7 +594,8 @@ func (c *CommandLine) DatabaseToken() (string, error) {
 	if err != nil {
 		return "", err
 	}
-	if response.Error() != nil || len((*response).Results[0].Series) == 0 {
+
+	if response.Error() != nil || len(response.Results) == 0 || len(response.Results[0].Series) == 0 {
 		return "", nil
 	}
 
@@ -645,7 +670,7 @@ func (c *CommandLine) writeColumns(response *client.Response, w io.Writer) {
 // formatResults will behave differently if you are formatting for columns or csv
 func (c *CommandLine) formatResults(result client.Result, separator string) []string {
 	rows := []string{}
-	// Create a tabbed writer for each result a they won't always line up
+	// Create a tabbed writer for each result as they won't always line up
 	for i, row := range result.Series {
 		// gather tags
 		tags := []string{}
@@ -676,15 +701,11 @@ func (c *CommandLine) formatResults(result client.Result, separator string) []st
 			rows = append(rows, "")
 		}
 
-		// If we are column format, we break out the name/tag to seperate lines
+		// If we are column format, we break out the name/tag to separate lines
 		if c.Format == "column" {
 			if row.Name != "" {
 				n := fmt.Sprintf("name: %s", row.Name)
 				rows = append(rows, n)
-				if len(tags) == 0 {
-					l := strings.Repeat("-", len(n))
-					rows = append(rows, l)
-				}
 			}
 			if len(tags) > 0 {
 				t := fmt.Sprintf("tags: %s", (strings.Join(tags, ", ")))
@@ -694,8 +715,8 @@ func (c *CommandLine) formatResults(result client.Result, separator string) []st
 
 		rows = append(rows, strings.Join(columnNames, separator))
 
-		// if format is column, break tags to their own line/format
-		if c.Format == "column" && len(tags) > 0 {
+		// if format is column, write dashes under each column
+		if c.Format == "column" {
 			lines := []string{}
 			for _, columnName := range columnNames {
 				lines = append(lines, strings.Repeat("-", len(columnName)))
@@ -719,7 +740,7 @@ func (c *CommandLine) formatResults(result client.Result, separator string) []st
 			}
 			rows = append(rows, strings.Join(values, separator))
 		}
-		// Outout a line separator if in column format
+		// Output a line separator if in column format
 		if c.Format == "column" {
 			rows = append(rows, "")
 		}
@@ -745,7 +766,7 @@ func interfaceToString(v interface{}) string {
 // Settings prints current settings
 func (c *CommandLine) Settings() {
 	w := new(tabwriter.Writer)
-	w.Init(os.Stdout, 0, 8, 1, '\t', 0)
+	w.Init(os.Stdout, 0, 1, 1, '\t', 0)
 	if c.Port > 0 {
 		fmt.Fprintf(w, "Host\t%s:%d\n", c.Host, c.Port)
 	} else {
