@@ -8,10 +8,10 @@ import (
 	"math/rand"
 	"os"
 	"os/signal"
-	"strings"
 	"syscall"
 	"time"
 
+	"github.com/influxdata/influxdb/cmd"
 	"github.com/influxdata/influxdb/cmd/influxd/backup"
 	"github.com/influxdata/influxdb/cmd/influxd/help"
 	"github.com/influxdata/influxdb/cmd/influxd/restore"
@@ -69,7 +69,7 @@ func NewMain() *Main {
 
 // Run determines and runs the command specified by the CLI args.
 func (m *Main) Run(args ...string) error {
-	name, args := ParseCommandName(args)
+	name, args := cmd.ParseCommandName(args)
 
 	// Extract name from args.
 	switch name {
@@ -141,31 +141,6 @@ func (m *Main) Run(args ...string) error {
 	return nil
 }
 
-// ParseCommandName extracts the command name and args from the args list.
-func ParseCommandName(args []string) (string, []string) {
-	// Retrieve command name as first argument.
-	var name string
-	if len(args) > 0 {
-		if !strings.HasPrefix(args[0], "-") {
-			name = args[0]
-		} else if args[0] == "-h" || args[0] == "-help" || args[0] == "--help" {
-			// Special case -h immediately following binary name
-			name = "help"
-		}
-	}
-
-	// If command is "help" and has an argument then rewrite args to use "-h".
-	if name == "help" && len(args) > 2 && !strings.HasPrefix(args[1], "-") {
-		return args[1], []string{"-h"}
-	}
-
-	// If a named command is specified then return it with its arguments.
-	if name != "" {
-		return name, args[1:]
-	}
-	return "", args
-}
-
 // VersionCommand represents the command executed by "influxd version".
 type VersionCommand struct {
 	Stdout io.Writer
