@@ -13,15 +13,13 @@ func TestConfig_Parse(t *testing.T) {
 	// Parse configuration.
 	var c run.Config
 	if err := c.FromToml(`
-join = "foo:123,bar:456"
-
 [meta]
 dir = "/tmp/meta"
 
 [data]
 dir = "/tmp/data"
 
-[cluster]
+[coordinator]
 
 [admin]
 bind-address = ":8083"
@@ -96,8 +94,6 @@ enabled = true
 		t.Fatalf("unexpected subscriber enabled: %v", c.Subscriber.Enabled)
 	} else if c.ContinuousQuery.Enabled != true {
 		t.Fatalf("unexpected continuous query enabled: %v", c.ContinuousQuery.Enabled)
-	} else if exp, got := "foo:123,bar:456", c.Join; exp != got {
-		t.Fatalf("unexpected join value: got %v, exp %v", got, exp)
 	}
 }
 
@@ -112,7 +108,7 @@ dir = "/tmp/meta"
 [data]
 dir = "/tmp/data"
 
-[cluster]
+[coordinator]
 
 [admin]
 bind-address = ":8083"
@@ -247,19 +243,15 @@ func TestConfig_DeprecatedOptions(t *testing.T) {
 	// Parse configuration.
 	var c run.Config
 	if err := c.FromToml(`
-[collectd]
-bind-address = ":1000"
-
-[opentsdb]
-bind-address = ":2000"
+[cluster]
+max-select-point = 100
 `); err != nil {
 		t.Fatal(err)
 	}
 
 	// Validate configuration.
-	if c.CollectdInputs[0].BindAddress != ":1000" {
-		t.Fatalf("unexpected collectd bind address: %s", c.CollectdInputs[0].BindAddress)
-	} else if c.OpenTSDBInputs[0].BindAddress != ":2000" {
-		t.Fatalf("unexpected opentsdb bind address: %s", c.OpenTSDBInputs[0].BindAddress)
+	if c.Coordinator.MaxSelectPointN != 100 {
+		t.Fatalf("unexpected coordinator max select points: %d", c.Coordinator.MaxSelectPointN)
+
 	}
 }
