@@ -8,6 +8,8 @@ import (
 	"sync"
 	"time"
 
+	"github.com/influxdata/influxdb/models"
+
 	"github.com/gogo/protobuf/proto"
 	internal "github.com/influxdata/influxdb/influxql/internal"
 )
@@ -17,11 +19,14 @@ var ErrUnknownCall = errors.New("unknown call")
 
 const (
 	// MinTime is used as the minimum time value when computing an unbounded range.
-	MinTime = int64(0)
+	// This time is one less than the MinNanoTime so that the first minimum
+	// time can be used as a sentinel value to signify that it is the default
+	// value rather than explicitly set by the user.
+	MinTime = models.MinNanoTime - 1
 
 	// MaxTime is used as the maximum time value when computing an unbounded range.
-	// This time is Jan 1, 2050 at midnight UTC.
-	MaxTime = int64(2524608000000000000)
+	// This time is 2262-04-11 23:47:16.854775806 +0000 UTC
+	MaxTime = models.MaxNanoTime
 )
 
 // Iterator represents a generic interface for all Iterators.
@@ -110,6 +115,38 @@ func (a Iterators) cast() interface{} {
 	return a
 }
 
+// Merge combines all iterators into a single iterator.
+// A sorted merge iterator or a merge iterator can be used based on opt.
+func (a Iterators) Merge(opt IteratorOptions) (Iterator, error) {
+	// Merge into a single iterator.
+	if opt.MergeSorted() {
+		itr := NewSortedMergeIterator(a, opt)
+		if itr != nil && opt.InterruptCh != nil {
+			itr = NewInterruptIterator(itr, opt.InterruptCh)
+		}
+		return itr, nil
+	}
+
+	itr := NewMergeIterator(a, opt)
+	if itr == nil {
+		return nil, nil
+	}
+
+	if opt.Expr != nil {
+		if expr, ok := opt.Expr.(*Call); ok && expr.Name == "count" {
+			opt.Expr = &Call{
+				Name: "sum",
+				Args: expr.Args,
+			}
+		}
+	}
+
+	if opt.InterruptCh != nil {
+		itr = NewInterruptIterator(itr, opt.InterruptCh)
+	}
+	return NewCallIterator(itr, opt)
+}
+
 // NewMergeIterator returns an iterator to merge itrs into one.
 // Inputs must either be merge iterators or only contain a single name/tag in
 // sorted order. The iterator will output all points by window, name/tag, then
@@ -117,8 +154,10 @@ func (a Iterators) cast() interface{} {
 // interval.
 func NewMergeIterator(inputs []Iterator, opt IteratorOptions) Iterator {
 	inputs = Iterators(inputs).filterNonNil()
-	if len(inputs) == 0 {
+	if n := len(inputs); n == 0 {
 		return nil
+	} else if n == 1 {
+		return inputs[0]
 	}
 
 	// Aggregate functions can use a more relaxed sorting so that points
@@ -137,6 +176,41 @@ func NewMergeIterator(inputs []Iterator, opt IteratorOptions) Iterator {
 	}
 }
 
+// NewParallelMergeIterator returns an iterator that breaks input iterators
+// into groups and processes them in parallel.
+func NewParallelMergeIterator(inputs []Iterator, opt IteratorOptions, parallelism int) Iterator {
+	inputs = Iterators(inputs).filterNonNil()
+	if len(inputs) == 0 {
+		return nil
+	} else if len(inputs) == 1 {
+		return inputs[0]
+	}
+
+	// Limit parallelism to the number of inputs.
+	if len(inputs) < parallelism {
+		parallelism = len(inputs)
+	}
+
+	// Determine the number of inputs per output iterator.
+	n := len(inputs) / parallelism
+
+	// Group iterators together.
+	outputs := make([]Iterator, parallelism)
+	for i := range outputs {
+		var slice []Iterator
+		if i < len(outputs)-1 {
+			slice = inputs[i*n : (i+1)*n]
+		} else {
+			slice = inputs[i*n:]
+		}
+
+		outputs[i] = newParallelIterator(NewMergeIterator(slice, opt))
+	}
+
+	// Merge all groups together.
+	return NewMergeIterator(outputs, opt)
+}
+
 // NewSortedMergeIterator returns an iterator to merge itrs into one.
 // Inputs must either be sorted merge iterators or only contain a single
 // name/tag in sorted order. The iterator will output all points by name/tag,
@@ -162,6 +236,26 @@ func NewSortedMergeIterator(inputs []Iterator, opt IteratorOptions) Iterator {
 	}
 }
 
+// newParallelIterator returns an iterator that runs in a separate goroutine.
+func newParallelIterator(input Iterator) Iterator {
+	if input == nil {
+		return nil
+	}
+
+	switch itr := input.(type) {
+	case FloatIterator:
+		return newFloatParallelIterator(itr)
+	case IntegerIterator:
+		return newIntegerParallelIterator(itr)
+	case StringIterator:
+		return newStringParallelIterator(itr)
+	case BooleanIterator:
+		return newBooleanParallelIterator(itr)
+	default:
+		panic(fmt.Sprintf("unsupported parallel iterator type: %T", itr))
+	}
+}
+
 // NewLimitIterator returns an iterator that limits the number of points per grouping.
 func NewLimitIterator(input Iterator, opt IteratorOptions) Iterator {
 	switch input := input.(type) {
@@ -245,7 +339,24 @@ func NewInterruptIterator(input Iterator, closing <-chan struct{}) Iterator {
 	case BooleanIterator:
 		return newBooleanInterruptIterator(input, closing)
 	default:
-		panic(fmt.Sprintf("unsupported fill iterator type: %T", input))
+		panic(fmt.Sprintf("unsupported interrupt iterator type: %T", input))
+	}
+}
+
+// NewCloseInterruptIterator returns an iterator that will invoke the Close() method on an
+// iterator when a channel has been closed.
+func NewCloseInterruptIterator(input Iterator, closing <-chan struct{}) Iterator {
+	switch input := input.(type) {
+	case FloatIterator:
+		return newFloatCloseInterruptIterator(input, closing)
+	case IntegerIterator:
+		return newIntegerCloseInterruptIterator(input, closing)
+	case StringIterator:
+		return newStringCloseInterruptIterator(input, closing)
+	case BooleanIterator:
+		return newBooleanCloseInterruptIterator(input, closing)
+	default:
+		panic(fmt.Sprintf("unsupported close iterator iterator type: %T", input))
 	}
 }
 
@@ -255,7 +366,7 @@ type AuxIterator interface {
 	IteratorCreator
 
 	// Auxilary iterator
-	Iterator(name string) Iterator
+	Iterator(name string, typ DataType) Iterator
 
 	// Start starts writing to the created iterators.
 	Start()
@@ -266,16 +377,16 @@ type AuxIterator interface {
 }
 
 // NewAuxIterator returns a new instance of AuxIterator.
-func NewAuxIterator(input Iterator, seriesKeys SeriesList, opt IteratorOptions) AuxIterator {
+func NewAuxIterator(input Iterator, opt IteratorOptions) AuxIterator {
 	switch input := input.(type) {
 	case FloatIterator:
-		return newFloatAuxIterator(input, seriesKeys, opt)
+		return newFloatAuxIterator(input, opt)
 	case IntegerIterator:
-		return newIntegerAuxIterator(input, seriesKeys, opt)
+		return newIntegerAuxIterator(input, opt)
 	case StringIterator:
-		return newStringAuxIterator(input, seriesKeys, opt)
+		return newStringAuxIterator(input, opt)
 	case BooleanIterator:
-		return newBooleanAuxIterator(input, seriesKeys, opt)
+		return newBooleanAuxIterator(input, opt)
 	default:
 		panic(fmt.Sprintf("unsupported aux iterator type: %T", input))
 	}
@@ -307,20 +418,10 @@ func (f *auxIteratorField) close() {
 type auxIteratorFields []*auxIteratorField
 
 // newAuxIteratorFields returns a new instance of auxIteratorFields from a list of field names.
-func newAuxIteratorFields(seriesKeys SeriesList, opt IteratorOptions) auxIteratorFields {
+func newAuxIteratorFields(opt IteratorOptions) auxIteratorFields {
 	fields := make(auxIteratorFields, len(opt.Aux))
-	for i, name := range opt.Aux {
-		fields[i] = &auxIteratorField{name: name, opt: opt}
-		for _, s := range seriesKeys {
-			aux := s.Aux[i]
-			if aux == Unknown {
-				continue
-			}
-
-			if fields[i].typ == Unknown || aux < fields[i].typ {
-				fields[i].typ = aux
-			}
-		}
+	for i, ref := range opt.Aux {
+		fields[i] = &auxIteratorField{name: ref.Val, typ: ref.Type, opt: opt}
 	}
 	return fields
 }
@@ -332,11 +433,11 @@ func (a auxIteratorFields) close() {
 }
 
 // iterator creates a new iterator for a named auxilary field.
-func (a auxIteratorFields) iterator(name string) Iterator {
+func (a auxIteratorFields) iterator(name string, typ DataType) Iterator {
 	for _, f := range a {
 		// Skip field if it's name doesn't match.
 		// Exit if no points were received by the iterator.
-		if f.name != name {
+		if f.name != name || (typ != Unknown && f.typ != typ) {
 			continue
 		}
 
@@ -350,7 +451,7 @@ func (a auxIteratorFields) iterator(name string) Iterator {
 			itr := &integerChanIterator{cond: sync.NewCond(&sync.Mutex{})}
 			f.append(itr)
 			return itr
-		case String:
+		case String, Tag:
 			itr := &stringChanIterator{cond: sync.NewCond(&sync.Mutex{})}
 			f.append(itr)
 			return itr
@@ -416,6 +517,7 @@ func (a auxIteratorFields) sendError(err error) {
 
 // DrainIterator reads all points from an iterator.
 func DrainIterator(itr Iterator) {
+	defer itr.Close()
 	switch itr := itr.(type) {
 	case FloatIterator:
 		for p, _ := itr.Next(); p != nil; p, _ = itr.Next() {
@@ -436,6 +538,7 @@ func DrainIterator(itr Iterator) {
 
 // DrainIterators reads all points from all iterators.
 func DrainIterators(itrs []Iterator) {
+	defer Iterators(itrs).Close()
 	for {
 		var hasData bool
 
@@ -470,18 +573,18 @@ func DrainIterators(itrs []Iterator) {
 }
 
 // NewReaderIterator returns an iterator that streams from a reader.
-func NewReaderIterator(r io.Reader, typ DataType, stats IteratorStats) (Iterator, error) {
+func NewReaderIterator(r io.Reader, typ DataType, stats IteratorStats) Iterator {
 	switch typ {
 	case Float:
-		return newFloatReaderIterator(r, stats), nil
+		return newFloatReaderIterator(r, stats)
 	case Integer:
-		return newIntegerReaderIterator(r, stats), nil
+		return newIntegerReaderIterator(r, stats)
 	case String:
-		return newStringReaderIterator(r, stats), nil
+		return newStringReaderIterator(r, stats)
 	case Boolean:
-		return newBooleanReaderIterator(r, stats), nil
+		return newBooleanReaderIterator(r, stats)
 	default:
-		return &nilFloatIterator{}, nil
+		return &nilFloatIterator{}
 	}
 }
 
@@ -491,10 +594,7 @@ type IteratorCreator interface {
 	CreateIterator(opt IteratorOptions) (Iterator, error)
 
 	// Returns the unique fields and dimensions across a list of sources.
-	FieldDimensions(sources Sources) (fields, dimensions map[string]struct{}, err error)
-
-	// Returns the series keys that will be returned by this iterator.
-	SeriesKeys(opt IteratorOptions) (SeriesList, error)
+	FieldDimensions(sources Sources) (fields map[string]DataType, dimensions map[string]struct{}, err error)
 
 	// Expands regex sources to all matching sources.
 	ExpandSources(sources Sources) (Sources, error)
@@ -523,6 +623,8 @@ func (a IteratorCreators) CreateIterator(opt IteratorOptions) (Iterator, error)
 			itr, err := ic.CreateIterator(opt)
 			if err != nil {
 				return err
+			} else if itr == nil {
+				continue
 			}
 			itrs = append(itrs, itr)
 		}
@@ -532,37 +634,16 @@ func (a IteratorCreators) CreateIterator(opt IteratorOptions) (Iterator, error)
 		return nil, err
 	}
 
-	// Merge into a single iterator.
-	if opt.MergeSorted() {
-		itr := NewSortedMergeIterator(itrs, opt)
-		if itr != nil && opt.InterruptCh != nil {
-			itr = NewInterruptIterator(itr, opt.InterruptCh)
-		}
-		return itr, nil
+	if len(itrs) == 0 {
+		return nil, nil
 	}
 
-	itr := NewMergeIterator(itrs, opt)
-	if itr != nil {
-		if opt.Expr != nil {
-			if expr, ok := opt.Expr.(*Call); ok && expr.Name == "count" {
-				opt.Expr = &Call{
-					Name: "sum",
-					Args: expr.Args,
-				}
-			}
-		}
-
-		if opt.InterruptCh != nil {
-			itr = NewInterruptIterator(itr, opt.InterruptCh)
-		}
-		return NewCallIterator(itr, opt)
-	}
-	return nil, nil
+	return Iterators(itrs).Merge(opt)
 }
 
 // FieldDimensions returns unique fields and dimensions from multiple iterator creators.
-func (a IteratorCreators) FieldDimensions(sources Sources) (fields, dimensions map[string]struct{}, err error) {
-	fields = make(map[string]struct{})
+func (a IteratorCreators) FieldDimensions(sources Sources) (fields map[string]DataType, dimensions map[string]struct{}, err error) {
+	fields = make(map[string]DataType)
 	dimensions = make(map[string]struct{})
 
 	for _, ic := range a {
@@ -570,8 +651,10 @@ func (a IteratorCreators) FieldDimensions(sources Sources) (fields, dimensions m
 		if err != nil {
 			return nil, nil, err
 		}
-		for k := range f {
-			fields[k] = struct{}{}
+		for k, typ := range f {
+			if _, ok := fields[k]; typ != Unknown && (!ok || typ < fields[k]) {
+				fields[k] = typ
+			}
 		}
 		for k := range d {
 			dimensions[k] = struct{}{}
@@ -580,35 +663,6 @@ func (a IteratorCreators) FieldDimensions(sources Sources) (fields, dimensions m
 	return
 }
 
-// SeriesKeys returns a list of series in all iterator creators in a.
-// If a series exists in multiple creators in a, all instances will be combined
-// into a single Series by calling Combine on it.
-func (a IteratorCreators) SeriesKeys(opt IteratorOptions) (SeriesList, error) {
-	seriesMap := make(map[string]Series)
-	for _, ic := range a {
-		series, err := ic.SeriesKeys(opt)
-		if err != nil {
-			return nil, err
-		}
-
-		for _, s := range series {
-			cur, ok := seriesMap[s.ID()]
-			if ok {
-				cur.Combine(&s)
-			} else {
-				seriesMap[s.ID()] = s
-			}
-		}
-	}
-
-	seriesList := make([]Series, 0, len(seriesMap))
-	for _, s := range seriesMap {
-		seriesList = append(seriesList, s)
-	}
-	sort.Sort(SeriesList(seriesList))
-	return SeriesList(seriesList), nil
-}
-
 // ExpandSources expands sources across all iterator creators and returns a unique result.
 func (a IteratorCreators) ExpandSources(sources Sources) (Sources, error) {
 	m := make(map[string]Source)
@@ -652,7 +706,7 @@ type IteratorOptions struct {
 	Expr Expr
 
 	// Auxilary tags or values to also retrieve for the point.
-	Aux []string
+	Aux []VarRef
 
 	// Data sources from which to retrieve data.
 	Sources []Source
@@ -725,7 +779,7 @@ func newIteratorOptionsStmt(stmt *SelectStatement, sopt *SelectOptions) (opt Ite
 	if interval < 0 {
 		interval = 0
 	} else if interval > 0 {
-		opt.Interval.Offset, err = stmt.GroupByOffset(&opt)
+		opt.Interval.Offset, err = stmt.GroupByOffset()
 		if err != nil {
 			return opt, err
 		}
@@ -789,7 +843,13 @@ func (opt IteratorOptions) Window(t int64) (start, end int64) {
 	t -= int64(opt.Interval.Offset)
 
 	// Truncate time by duration.
-	t -= t % int64(opt.Interval.Duration)
+	dt := t % int64(opt.Interval.Duration)
+	if dt < 0 {
+		// Negative modulo rounds up instead of down, so offset
+		// with the duration.
+		dt += int64(opt.Interval.Duration)
+	}
+	t -= dt
 
 	// Apply the offset.
 	start = t + int64(opt.Interval.Offset)
@@ -845,7 +905,6 @@ func (opt *IteratorOptions) UnmarshalBinary(buf []byte) error {
 
 func encodeIteratorOptions(opt *IteratorOptions) *internal.IteratorOptions {
 	pb := &internal.IteratorOptions{
-		Aux:        opt.Aux,
 		Interval:   encodeInterval(opt.Interval),
 		Dimensions: opt.Dimensions,
 		Fill:       proto.Int32(int32(opt.Fill)),
@@ -864,6 +923,14 @@ func encodeIteratorOptions(opt *IteratorOptions) *internal.IteratorOptions {
 		pb.Expr = proto.String(opt.Expr.String())
 	}
 
+	// Convert and encode aux fields as variable references.
+	pb.Fields = make([]*internal.VarRef, len(opt.Aux))
+	pb.Aux = make([]string, len(opt.Aux))
+	for i, ref := range opt.Aux {
+		pb.Fields[i] = encodeVarRef(ref)
+		pb.Aux[i] = ref.Val
+	}
+
 	// Convert and encode sources to measurements.
 	sources := make([]*internal.Measurement, len(opt.Sources))
 	for i, source := range opt.Sources {
@@ -887,7 +954,6 @@ func encodeIteratorOptions(opt *IteratorOptions) *internal.IteratorOptions {
 
 func decodeIteratorOptions(pb *internal.IteratorOptions) (*IteratorOptions, error) {
 	opt := &IteratorOptions{
-		Aux:        pb.GetAux(),
 		Interval:   decodeInterval(pb.GetInterval()),
 		Dimensions: pb.GetDimensions(),
 		Fill:       FillOption(pb.GetFill()),
@@ -911,7 +977,20 @@ func decodeIteratorOptions(pb *internal.IteratorOptions) (*IteratorOptions, erro
 		opt.Expr = expr
 	}
 
-	// Convert and encode sources to measurements.
+	// Convert and decode variable references.
+	if fields := pb.GetFields(); fields != nil {
+		opt.Aux = make([]VarRef, len(fields))
+		for i, ref := range fields {
+			opt.Aux[i] = decodeVarRef(ref)
+		}
+	} else {
+		opt.Aux = make([]VarRef, len(pb.GetAux()))
+		for i, name := range pb.GetAux() {
+			opt.Aux[i] = VarRef{Val: name}
+		}
+	}
+
+	// Convert and dencode sources to measurements.
 	sources := make([]Source, len(pb.GetSources()))
 	for i, source := range pb.GetSources() {
 		mm, err := decodeMeasurement(source)
@@ -962,110 +1041,6 @@ func (v *selectInfo) Visit(n Node) Visitor {
 	return v
 }
 
-// Series represents a series that will be returned by the iterator.
-type Series struct {
-	Name string
-	Tags Tags
-	Aux  []DataType
-}
-
-// ID is a single string that combines the name and tags id for the series.
-func (s *Series) ID() string {
-	return s.Name + "\x00" + s.Tags.ID()
-}
-
-// Combine combines two series with the same name and tags.
-// It will promote auxiliary iterator types to the highest type.
-func (s *Series) Combine(other *Series) {
-	for i, t := range s.Aux {
-		if other.Aux[i] == Unknown {
-			continue
-		}
-
-		if t == Unknown || other.Aux[i] < t {
-			s.Aux[i] = other.Aux[i]
-		}
-	}
-}
-
-func encodeSeries(s Series) *internal.Series {
-	aux := make([]uint32, len(s.Aux))
-	for i := range s.Aux {
-		aux[i] = uint32(s.Aux[i])
-	}
-
-	return &internal.Series{
-		Name: proto.String(s.Name),
-		Tags: encodeTags(s.Tags.KeyValues()),
-		Aux:  aux,
-	}
-}
-
-func decodeSeries(pb *internal.Series) Series {
-	var aux []DataType
-	if len(pb.GetAux()) > 0 {
-		aux = make([]DataType, len(pb.GetAux()))
-		for i := range pb.GetAux() {
-			aux[i] = DataType(pb.GetAux()[i])
-		}
-	}
-
-	return Series{
-		Name: pb.GetName(),
-		Tags: newTagsID(string(pb.GetTags())),
-		Aux:  aux,
-	}
-}
-
-// SeriesList is a list of series that will be returned by an iterator.
-type SeriesList []Series
-
-func (a SeriesList) Len() int      { return len(a) }
-func (a SeriesList) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
-
-func (a SeriesList) Less(i, j int) bool {
-	if a[i].Name != a[j].Name {
-		return a[i].Name < a[j].Name
-	}
-	return a[i].Tags.ID() < a[j].Tags.ID()
-}
-
-// MarshalBinary encodes list into a binary format.
-func (a SeriesList) MarshalBinary() ([]byte, error) {
-	return proto.Marshal(encodeSeriesList(a))
-}
-
-// UnmarshalBinary decodes from a binary format.
-func (a *SeriesList) UnmarshalBinary(buf []byte) error {
-	var pb internal.SeriesList
-	if err := proto.Unmarshal(buf, &pb); err != nil {
-		return err
-	}
-
-	(*a) = decodeSeriesList(&pb)
-
-	return nil
-}
-
-func encodeSeriesList(a SeriesList) *internal.SeriesList {
-	pb := make([]*internal.Series, len(a))
-	for i := range a {
-		pb[i] = encodeSeries(a[i])
-	}
-
-	return &internal.SeriesList{
-		Items: pb,
-	}
-}
-
-func decodeSeriesList(pb *internal.SeriesList) SeriesList {
-	a := make([]Series, len(pb.GetItems()))
-	for i := range pb.GetItems() {
-		a[i] = decodeSeries(pb.GetItems()[i])
-	}
-	return SeriesList(a)
-}
-
 // Interval represents a repeating interval for a query.
 type Interval struct {
 	Duration time.Duration
@@ -1089,6 +1064,20 @@ func decodeInterval(pb *internal.Interval) Interval {
 	}
 }
 
+func encodeVarRef(ref VarRef) *internal.VarRef {
+	return &internal.VarRef{
+		Val:  proto.String(ref.Val),
+		Type: proto.Int32(int32(ref.Type)),
+	}
+}
+
+func decodeVarRef(pb *internal.VarRef) VarRef {
+	return VarRef{
+		Val:  pb.GetVal(),
+		Type: DataType(pb.GetType()),
+	}
+}
+
 type nilFloatIterator struct{}
 
 func (*nilFloatIterator) Stats() IteratorStats       { return IteratorStats{} }
@@ -1207,7 +1196,11 @@ func (itr *floatFastDedupeIterator) Next() (*FloatPoint, error) {
 		}
 
 		// If the point has already been output then move to the next point.
-		key := fastDedupeKey{p.Name, p.Aux[0]}
+		key := fastDedupeKey{name: p.Name}
+		key.values[0] = p.Aux[0]
+		if len(p.Aux) > 1 {
+			key.values[1] = p.Aux[1]
+		}
 		if _, ok := itr.m[key]; ok {
 			continue
 		}
@@ -1219,8 +1212,8 @@ func (itr *floatFastDedupeIterator) Next() (*FloatPoint, error) {
 }
 
 type fastDedupeKey struct {
-	name  string
-	value interface{}
+	name   string
+	values [2]interface{}
 }
 
 type reverseStringSlice []string
