@@ -15,7 +15,15 @@ import (
 )
 
 var (
-	tags       = models.Tags{"foo": "bar", "apple": "orange", "host": "serverA", "region": "uswest"}
+	tags   = models.NewTags(map[string]string{"foo": "bar", "apple": "orange", "host": "serverA", "region": "uswest"})
+	fields = models.Fields{
+		"int64":         int64(math.MaxInt64),
+		"uint32":        uint32(math.MaxUint32),
+		"string":        "String field that has a decent length, probably some log message or something",
+		"boolean":       false,
+		"float64-tiny":  float64(math.SmallestNonzeroFloat64),
+		"float64-large": float64(math.MaxFloat64),
+	}
 	maxFloat64 = strconv.FormatFloat(math.MaxFloat64, 'f', 1, 64)
 	minFloat64 = strconv.FormatFloat(-math.MaxFloat64, 'f', 1, 64)
 )
@@ -35,6 +43,72 @@ func BenchmarkMarshal(b *testing.B) {
 	}
 }
 
+func TestPoint_StringSize(t *testing.T) {
+	testPoint_cube(t, func(p models.Point) {
+		l := p.StringSize()
+		s := p.String()
+
+		if l != len(s) {
+			t.Errorf("Incorrect length for %q. got %v, exp %v", s, l, len(s))
+		}
+	})
+
+}
+
+func TestPoint_AppendString(t *testing.T) {
+	testPoint_cube(t, func(p models.Point) {
+		got := p.AppendString(nil)
+		exp := []byte(p.String())
+
+		if !reflect.DeepEqual(exp, got) {
+			t.Errorf("AppendString() didn't match String(): got %v, exp %v", got, exp)
+		}
+	})
+}
+
+func testPoint_cube(t *testing.T, f func(p models.Point)) {
+	// heard of a table-driven test? let's make a cube-driven test...
+	tagList := []models.Tags{nil, {models.Tag{Key: []byte("foo"), Value: []byte("bar")}}, tags}
+	fieldList := []models.Fields{{"a": 42.0}, {"a": 42, "b": "things"}, fields}
+	timeList := []time.Time{time.Time{}, time.Unix(0, 0), time.Unix(-34526, 0), time.Unix(231845, 0), time.Now()}
+
+	for _, tagSet := range tagList {
+		for _, fieldSet := range fieldList {
+			for _, pointTime := range timeList {
+				p, err := models.NewPoint("test", tagSet, fieldSet, pointTime)
+				if err != nil {
+					t.Errorf("unexpected error creating point: %v", err)
+					continue
+				}
+
+				f(p)
+			}
+		}
+	}
+}
+
+var p models.Point
+
+func BenchmarkNewPoint(b *testing.B) {
+	ts := time.Now()
+	for i := 0; i < b.N; i++ {
+		p, _ = models.NewPoint("measurement", tags, fields, ts)
+	}
+}
+
+func BenchmarkParsePointNoTags5000(b *testing.B) {
+	var batch [5000]string
+	for i := 0; i < len(batch); i++ {
+		batch[i] = `cpu value=1i 1000000000`
+	}
+	lines := strings.Join(batch[:], "\n")
+	b.ResetTimer()
+	for i := 0; i < b.N; i++ {
+		models.ParsePoints([]byte(lines))
+		b.SetBytes(int64(len(lines)))
+	}
+}
+
 func BenchmarkParsePointNoTags(b *testing.B) {
 	line := `cpu value=1i 1000000000`
 	for i := 0; i < b.N; i++ {
@@ -115,7 +189,7 @@ func BenchmarkParsePointsTagsUnSorted10(b *testing.B) {
 func BenchmarkParseKey(b *testing.B) {
 	line := `cpu,region=us-west,host=serverA,env=prod,target=servers,zone=1c,tag1=value1,tag2=value2,tag3=value3,tag4=value4,tag5=value5`
 	for i := 0; i < b.N; i++ {
-		models.ParseKey(line)
+		models.ParseKey([]byte(line))
 	}
 }
 
@@ -163,9 +237,9 @@ func test(t *testing.T, line string, point TestPoint) {
 		t.Errorf(`ParsePoints("%s") tags mismatch. got %v, exp %v`, line, pts[0].Tags(), exp)
 	}
 
-	for tag, value := range pts[0].Tags() {
-		if value != point.RawTags[tag] {
-			t.Errorf(`ParsePoints("%s") tags mismatch. got %v, exp %v`, line, value, point.RawTags[tag])
+	for _, tag := range pts[0].Tags() {
+		if !bytes.Equal(tag.Value, point.RawTags.Get(tag.Key)) {
+			t.Errorf(`ParsePoints("%s") tags mismatch. got %s, exp %s`, line, tag.Value, point.RawTags.Get(tag.Key))
 		}
 	}
 
@@ -639,7 +713,7 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `foo\,bar value=1i`,
 		NewTestPoint(
 			"foo,bar", // comma in the name
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": int64(1),
 			},
@@ -649,9 +723,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu\,main,regions=east value=1.0`,
 		NewTestPoint(
 			"cpu,main", // comma in the name
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -661,9 +735,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu\ load,region=east value=1.0`,
 		NewTestPoint(
 			"cpu load", // space in the name
-			models.Tags{
+			models.NewTags(map[string]string{
 				"region": "east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -673,9 +747,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu\=load,region=east value=1.0`,
 		NewTestPoint(
 			`cpu\=load`, // backslash is literal
-			models.Tags{
+			models.NewTags(map[string]string{
 				"region": "east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -685,9 +759,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu=load,region=east value=1.0`,
 		NewTestPoint(
 			`cpu=load`, // literal equals is fine in measurement name
-			models.Tags{
+			models.NewTags(map[string]string{
 				"region": "east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -696,9 +770,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// commas in tag names
 	test(t, `cpu,region\,zone=east value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"region,zone": "east", // comma in the tag key
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -707,9 +781,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// spaces in tag name
 	test(t, `cpu,region\ zone=east value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"region zone": "east", // space in the tag name
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -718,9 +792,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// backslash with escaped equals in tag name
 	test(t, `cpu,reg\\=ion=east value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				`reg\=ion`: "east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -729,9 +803,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// space is tag name
 	test(t, `cpu,\ =east value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				" ": "east", // tag name is single space
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -740,9 +814,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// commas in tag values
 	test(t, `cpu,regions=east\,west value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east,west", // comma in the tag value
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -752,9 +826,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu,regions=\\ east value=1.0`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": `\ east`,
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -764,9 +838,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu,regions=eas\\ t value=1.0`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": `eas\ t`,
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -776,9 +850,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu,regions=east\\  value=1.0`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": `east\ `,
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -787,9 +861,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// spaces in tag values
 	test(t, `cpu,regions=east\ west value=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east west", // comma in the tag value
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -798,9 +872,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// commas in field keys
 	test(t, `cpu,regions=east value\,ms=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east",
-			},
+			}),
 			models.Fields{
 				"value,ms": 1.0, // comma in the field keys
 			},
@@ -809,9 +883,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// spaces in field keys
 	test(t, `cpu,regions=east value\ ms=1.0`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east",
-			},
+			}),
 			models.Fields{
 				"value ms": 1.0, // comma in the field keys
 			},
@@ -820,10 +894,10 @@ func TestParsePointUnescape(t *testing.T) {
 	// tag with no value
 	test(t, `cpu,regions=east value="1"`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east",
 				"foobar":  "",
-			},
+			}),
 			models.Fields{
 				"value": "1",
 			},
@@ -832,9 +906,9 @@ func TestParsePointUnescape(t *testing.T) {
 	// commas in field values
 	test(t, `cpu,regions=east value="1,0"`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "east",
-			},
+			}),
 			models.Fields{
 				"value": "1,0", // comma in the field value
 			},
@@ -844,9 +918,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu,regions=eas\t value=1.0`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": "eas\\t",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -856,9 +930,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu,regions=\\,\,\=east value=1.0`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"regions": `\,,=east`,
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -868,7 +942,7 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu \a=1i`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"\\a": int64(1), // Left as parsed since it's not a known escape sequence.
 			},
@@ -878,9 +952,9 @@ func TestParsePointUnescape(t *testing.T) {
 	test(t, `cpu=load,equals\=foo=tag\=value value=1i`,
 		NewTestPoint(
 			"cpu=load", // Not escaped
-			models.Tags{
+			models.NewTags(map[string]string{
 				"equals=foo": "tag=value", // Tag and value unescaped
-			},
+			}),
 			models.Fields{
 				"value": int64(1),
 			},
@@ -892,7 +966,7 @@ func TestParsePointWithTags(t *testing.T) {
 	test(t,
 		"cpu,host=serverA,region=us-east value=1.0 1000000000",
 		NewTestPoint("cpu",
-			models.Tags{"host": "serverA", "region": "us-east"},
+			models.NewTags(map[string]string{"host": "serverA", "region": "us-east"}),
 			models.Fields{"value": 1.0}, time.Unix(1, 0)))
 }
 
@@ -924,10 +998,10 @@ func TestParsePointWithDuplicateTags(t *testing.T) {
 func TestParsePointWithStringField(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value=1.0,str="foo",str2="bar" 1000000000`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   "foo",
@@ -938,10 +1012,10 @@ func TestParsePointWithStringField(t *testing.T) {
 
 	test(t, `cpu,host=serverA,region=us-east str="foo \" bar" 1000000000`,
 		NewTestPoint("cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"str": `foo " bar`,
 			},
@@ -954,10 +1028,10 @@ func TestParsePointWithStringWithSpaces(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value=1.0,str="foo bar" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   "foo bar", // spaces in string value
@@ -970,10 +1044,10 @@ func TestParsePointWithStringWithNewline(t *testing.T) {
 	test(t, "cpu,host=serverA,region=us-east value=1.0,str=\"foo\nbar\" 1000000000",
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   "foo\nbar", // newline in string value
@@ -987,10 +1061,10 @@ func TestParsePointWithStringWithCommas(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value=1.0,str="foo\,bar" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   `foo\,bar`, // commas in string value
@@ -1002,16 +1076,31 @@ func TestParsePointWithStringWithCommas(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value=1.0,str="foo,bar" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   "foo,bar", // commas in string value
 			},
 			time.Unix(1, 0)),
 	)
+
+	// string w/ trailing escape chars
+	test(t, `cpu,host=serverA,region=us-east str="foo\\",str2="bar" 1000000000`,
+		NewTestPoint(
+			"cpu",
+			models.NewTags(map[string]string{
+				"host":   "serverA",
+				"region": "us-east",
+			}),
+			models.Fields{
+				"str":  "foo\\", // trailing escape char
+				"str2": "bar",
+			},
+			time.Unix(1, 0)),
+	)
 }
 
 func TestParsePointQuotedMeasurement(t *testing.T) {
@@ -1019,10 +1108,10 @@ func TestParsePointQuotedMeasurement(t *testing.T) {
 	test(t, `"cpu",host=serverA,region=us-east value=1.0 1000000000`,
 		NewTestPoint(
 			`"cpu"`,
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1034,10 +1123,10 @@ func TestParsePointQuotedTags(t *testing.T) {
 	test(t, `cpu,"host"="serverA",region=us-east value=1.0 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				`"host"`: `"serverA"`,
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1056,7 +1145,7 @@ func TestParsePointsUnbalancedQuotedTags(t *testing.T) {
 	}
 
 	// Expected " in the tag value
-	exp := models.MustNewPoint("baz", models.Tags{"mytag": `"a`},
+	exp := models.MustNewPoint("baz", models.NewTags(map[string]string{"mytag": `"a`}),
 		models.Fields{"x": float64(1)}, time.Unix(0, 1441103862125))
 
 	if pts[0].String() != exp.String() {
@@ -1064,7 +1153,7 @@ func TestParsePointsUnbalancedQuotedTags(t *testing.T) {
 	}
 
 	// Expected two points to ensure we did not overscan the line
-	exp = models.MustNewPoint("baz", models.Tags{"mytag": `a`},
+	exp = models.MustNewPoint("baz", models.NewTags(map[string]string{"mytag": `a`}),
 		models.Fields{"z": float64(1)}, time.Unix(0, 1441103862126))
 
 	if pts[1].String() != exp.String() {
@@ -1078,10 +1167,10 @@ func TestParsePointEscapedStringsAndCommas(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value="{Hello\"{,}\" World}" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": `{Hello"{,}" World}`,
 			},
@@ -1092,10 +1181,10 @@ func TestParsePointEscapedStringsAndCommas(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value="{Hello\"{\,}\" World}" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": `{Hello"{\,}" World}`,
 			},
@@ -1107,10 +1196,10 @@ func TestParsePointWithStringWithEquals(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east str="foo=bar",value=1.0 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": 1.0,
 				"str":   "foo=bar", // spaces in string value
@@ -1123,7 +1212,7 @@ func TestParsePointWithStringWithBackslash(t *testing.T) {
 	test(t, `cpu value="test\\\"" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": `test\"`,
 			},
@@ -1133,7 +1222,7 @@ func TestParsePointWithStringWithBackslash(t *testing.T) {
 	test(t, `cpu value="test\\" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": `test\`,
 			},
@@ -1143,7 +1232,7 @@ func TestParsePointWithStringWithBackslash(t *testing.T) {
 	test(t, `cpu value="test\\\"" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": `test\"`,
 			},
@@ -1153,7 +1242,7 @@ func TestParsePointWithStringWithBackslash(t *testing.T) {
 	test(t, `cpu value="test\"" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": `test"`,
 			},
@@ -1165,10 +1254,10 @@ func TestParsePointWithBoolField(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east true=true,t=t,T=T,TRUE=TRUE,True=True,false=false,f=f,F=F,FALSE=FALSE,False=False 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"t":     true,
 				"T":     true,
@@ -1189,10 +1278,10 @@ func TestParsePointUnicodeString(t *testing.T) {
 	test(t, `cpu,host=serverA,region=us-east value="wè" 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{
+			models.NewTags(map[string]string{
 				"host":   "serverA",
 				"region": "us-east",
-			},
+			}),
 			models.Fields{
 				"value": "wè",
 			},
@@ -1204,7 +1293,7 @@ func TestParsePointNegativeTimestamp(t *testing.T) {
 	test(t, `cpu value=1 -1`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1216,7 +1305,7 @@ func TestParsePointMaxTimestamp(t *testing.T) {
 	test(t, fmt.Sprintf(`cpu value=1 %d`, models.MaxNanoTime),
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1228,7 +1317,7 @@ func TestParsePointMinTimestamp(t *testing.T) {
 	test(t, `cpu value=1 -9223372036854775806`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1259,7 +1348,7 @@ func TestNewPointFloatWithoutDecimal(t *testing.T) {
 	test(t, `cpu value=1 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1270,7 +1359,7 @@ func TestNewPointNegativeFloat(t *testing.T) {
 	test(t, `cpu value=-0.64 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": -0.64,
 			},
@@ -1282,7 +1371,7 @@ func TestNewPointFloatNoDecimal(t *testing.T) {
 	test(t, `cpu value=1. 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": 1.0,
 			},
@@ -1294,7 +1383,7 @@ func TestNewPointFloatScientific(t *testing.T) {
 	test(t, `cpu value=6.632243e+06 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": float64(6632243),
 			},
@@ -1306,7 +1395,7 @@ func TestNewPointLargeInteger(t *testing.T) {
 	test(t, `cpu value=6632243i 1000000000`,
 		NewTestPoint(
 			"cpu",
-			models.Tags{},
+			models.NewTags(map[string]string{}),
 			models.Fields{
 				"value": int64(6632243), // if incorrectly encoded as a float, it would show up as 6.632243e+06
 			},
@@ -1403,7 +1492,7 @@ func TestParsePointToString(t *testing.T) {
 		t.Errorf("ParsePoint() to string mismatch:\n got %v\n exp %v", got, line)
 	}
 
-	pt = models.MustNewPoint("cpu", models.Tags{"host": "serverA", "region": "us-east"},
+	pt = models.MustNewPoint("cpu", models.NewTags(map[string]string{"host": "serverA", "region": "us-east"}),
 		models.Fields{"int": 10, "float": float64(11.0), "float2": float64(12.123), "bool": false, "str": "string val"},
 		time.Unix(1, 0))
 
@@ -1600,26 +1689,26 @@ cpu,host=serverA,region=us-east value=1.0 946730096789012345`,
 
 func TestNewPointEscaped(t *testing.T) {
 	// commas
-	pt := models.MustNewPoint("cpu,main", models.Tags{"tag,bar": "value"}, models.Fields{"name,bar": 1.0}, time.Unix(0, 0))
+	pt := models.MustNewPoint("cpu,main", models.NewTags(map[string]string{"tag,bar": "value"}), models.Fields{"name,bar": 1.0}, time.Unix(0, 0))
 	if exp := `cpu\,main,tag\,bar=value name\,bar=1 0`; pt.String() != exp {
 		t.Errorf("NewPoint().String() mismatch.\ngot %v\nexp %v", pt.String(), exp)
 	}
 
 	// spaces
-	pt = models.MustNewPoint("cpu main", models.Tags{"tag bar": "value"}, models.Fields{"name bar": 1.0}, time.Unix(0, 0))
+	pt = models.MustNewPoint("cpu main", models.NewTags(map[string]string{"tag bar": "value"}), models.Fields{"name bar": 1.0}, time.Unix(0, 0))
 	if exp := `cpu\ main,tag\ bar=value name\ bar=1 0`; pt.String() != exp {
 		t.Errorf("NewPoint().String() mismatch.\ngot %v\nexp %v", pt.String(), exp)
 	}
 
 	// equals
-	pt = models.MustNewPoint("cpu=main", models.Tags{"tag=bar": "value=foo"}, models.Fields{"name=bar": 1.0}, time.Unix(0, 0))
+	pt = models.MustNewPoint("cpu=main", models.NewTags(map[string]string{"tag=bar": "value=foo"}), models.Fields{"name=bar": 1.0}, time.Unix(0, 0))
 	if exp := `cpu=main,tag\=bar=value\=foo name\=bar=1 0`; pt.String() != exp {
 		t.Errorf("NewPoint().String() mismatch.\ngot %v\nexp %v", pt.String(), exp)
 	}
 }
 
 func TestNewPointWithoutField(t *testing.T) {
-	_, err := models.NewPoint("cpu", models.Tags{"tag": "bar"}, models.Fields{}, time.Unix(0, 0))
+	_, err := models.NewPoint("cpu", models.NewTags(map[string]string{"tag": "bar"}), models.Fields{}, time.Unix(0, 0))
 	if err == nil {
 		t.Fatalf(`NewPoint() expected error. got nil`)
 	}
@@ -1645,19 +1734,19 @@ func TestNewPointUnhandledType(t *testing.T) {
 }
 
 func TestMakeKeyEscaped(t *testing.T) {
-	if exp, got := `cpu\ load`, models.MakeKey([]byte(`cpu\ load`), models.Tags{}); string(got) != exp {
+	if exp, got := `cpu\ load`, models.MakeKey([]byte(`cpu\ load`), models.NewTags(map[string]string{})); string(got) != exp {
 		t.Errorf("MakeKey() mismatch.\ngot %v\nexp %v", got, exp)
 	}
 
-	if exp, got := `cpu\ load`, models.MakeKey([]byte(`cpu load`), models.Tags{}); string(got) != exp {
+	if exp, got := `cpu\ load`, models.MakeKey([]byte(`cpu load`), models.NewTags(map[string]string{})); string(got) != exp {
 		t.Errorf("MakeKey() mismatch.\ngot %v\nexp %v", got, exp)
 	}
 
-	if exp, got := `cpu\,load`, models.MakeKey([]byte(`cpu\,load`), models.Tags{}); string(got) != exp {
+	if exp, got := `cpu\,load`, models.MakeKey([]byte(`cpu\,load`), models.NewTags(map[string]string{})); string(got) != exp {
 		t.Errorf("MakeKey() mismatch.\ngot %v\nexp %v", got, exp)
 	}
 
-	if exp, got := `cpu\,load`, models.MakeKey([]byte(`cpu,load`), models.Tags{}); string(got) != exp {
+	if exp, got := `cpu\,load`, models.MakeKey([]byte(`cpu,load`), models.NewTags(map[string]string{})); string(got) != exp {
 		t.Errorf("MakeKey() mismatch.\ngot %v\nexp %v", got, exp)
 	}
 
@@ -1880,7 +1969,202 @@ func TestNewPointsRejectsMaxKey(t *testing.T) {
 }
 
 func TestParseKeyEmpty(t *testing.T) {
-	if _, _, err := models.ParseKey(""); err != nil {
+	if _, _, err := models.ParseKey(nil); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 }
+
+func TestPoint_FieldIterator_Simple(t *testing.T) {
+
+	p, err := models.ParsePoints([]byte(`m v=42i,f=42 36`))
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if len(p) != 1 {
+		t.Fatalf("wrong number of points, got %d, exp %d", len(p), 1)
+	}
+
+	fi := p[0].FieldIterator()
+
+	if !fi.Next() {
+		t.Fatal("field iterator terminated before first field")
+	}
+
+	if fi.Type() != models.Integer {
+		t.Fatalf("'42i' should be an Integer, got %v", fi.Type())
+	}
+
+	if fi.IntegerValue() != 42 {
+		t.Fatalf("'42i' should be 42, got %d", fi.IntegerValue())
+	}
+
+	if !fi.Next() {
+		t.Fatalf("field iterator terminated before second field")
+	}
+
+	if fi.Type() != models.Float {
+		t.Fatalf("'42' should be a Float, got %v", fi.Type())
+	}
+
+	if fi.FloatValue() != 42.0 {
+		t.Fatalf("'42' should be %f, got %f", 42.0, fi.FloatValue())
+	}
+
+	if fi.Next() {
+		t.Fatal("field iterator didn't terminate")
+	}
+}
+
+func toFields(fi models.FieldIterator) models.Fields {
+	m := make(models.Fields)
+	for fi.Next() {
+		var v interface{}
+		switch fi.Type() {
+		case models.Float:
+			v = fi.FloatValue()
+		case models.Integer:
+			v = fi.IntegerValue()
+		case models.String:
+			v = fi.StringValue()
+		case models.Boolean:
+			v = fi.BooleanValue()
+		case models.Empty:
+			v = nil
+		default:
+			panic("unknown type")
+		}
+		m[string(fi.FieldKey())] = v
+	}
+	return m
+}
+
+func TestPoint_FieldIterator_FieldMap(t *testing.T) {
+
+	points, err := models.ParsePointsString(`
+m v=42
+m v=42i
+m v="string"
+m v=true
+m v="string\"with\"escapes"
+m v=42i,f=42,g=42.314
+m a=2i,b=3i,c=true,d="stuff",e=-0.23,f=123.456
+`)
+
+	if err != nil {
+		t.Fatal("failed to parse test points:", err)
+	}
+
+	for _, p := range points {
+		exp := p.Fields()
+		got := toFields(p.FieldIterator())
+
+		if !reflect.DeepEqual(got, exp) {
+			t.Errorf("FieldIterator failed for %#q: got %#v, exp %#v", p.String(), got, exp)
+		}
+	}
+}
+
+func TestPoint_FieldIterator_Delete_Begin(t *testing.T) {
+	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
+	if err != nil || len(points) != 1 {
+		t.Fatal("failed parsing point")
+	}
+
+	fi := points[0].FieldIterator()
+	fi.Next() // a
+	fi.Delete()
+
+	fi.Reset()
+
+	got := toFields(fi)
+	exp := models.Fields{"b": float64(2), "c": float64(3)}
+
+	if !reflect.DeepEqual(got, exp) {
+		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+	}
+}
+
+func TestPoint_FieldIterator_Delete_Middle(t *testing.T) {
+	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
+	if err != nil || len(points) != 1 {
+		t.Fatal("failed parsing point")
+	}
+
+	fi := points[0].FieldIterator()
+	fi.Next() // a
+	fi.Next() // b
+	fi.Delete()
+
+	fi.Reset()
+
+	got := toFields(fi)
+	exp := models.Fields{"a": float64(1), "c": float64(3)}
+
+	if !reflect.DeepEqual(got, exp) {
+		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+	}
+}
+
+func TestPoint_FieldIterator_Delete_End(t *testing.T) {
+	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
+	if err != nil || len(points) != 1 {
+		t.Fatal("failed parsing point")
+	}
+
+	fi := points[0].FieldIterator()
+	fi.Next() // a
+	fi.Next() // b
+	fi.Next() // c
+	fi.Delete()
+
+	fi.Reset()
+
+	got := toFields(fi)
+	exp := models.Fields{"a": float64(1), "b": float64(2)}
+
+	if !reflect.DeepEqual(got, exp) {
+		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+	}
+}
+
+func TestPoint_FieldIterator_Delete_Nothing(t *testing.T) {
+	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
+	if err != nil || len(points) != 1 {
+		t.Fatal("failed parsing point")
+	}
+
+	fi := points[0].FieldIterator()
+	fi.Delete()
+
+	fi.Reset()
+
+	got := toFields(fi)
+	exp := models.Fields{"a": float64(1), "b": float64(2), "c": float64(3)}
+
+	if !reflect.DeepEqual(got, exp) {
+		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+	}
+}
+
+func TestPoint_FieldIterator_Delete_Twice(t *testing.T) {
+	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
+	if err != nil || len(points) != 1 {
+		t.Fatal("failed parsing point")
+	}
+
+	fi := points[0].FieldIterator()
+	fi.Next() // a
+	fi.Next() // b
+	fi.Delete()
+	fi.Delete() // no-op
+
+	fi.Reset()
+
+	got := toFields(fi)
+	exp := models.Fields{"a": float64(1), "c": float64(3)}
+
+	if !reflect.DeepEqual(got, exp) {
+		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+	}
+}
