@@ -3,7 +3,7 @@ package continuous_querier
 import (
 	"time"
 
-	"github.com/influxdb/influxdb/toml"
+	"github.com/influxdata/influxdb/toml"
 )
 
 // Default values for aspects of interval computation.
