@@ -2,6 +2,9 @@ package subscriber
 
 import (
 	"errors"
+	"fmt"
+	"os"
+	"path/filepath"
 	"time"
 
 	"github.com/influxdata/influxdb/toml"
@@ -20,6 +23,14 @@ type Config struct {
 
 	HTTPTimeout toml.Duration `toml:"http-timeout"`
 
+	// InsecureSkipVerify gets passed to the http client, if true, it will
+	// skip https certificate verification. Defaults to false
+	InsecureSkipVerify bool `toml:"insecure-skip-verify"`
+
+	// configure the path to the PEM encoded CA certs file. If the
+	// empty string, the default system certs will be used
+	CaCerts string `toml:"ca-certs"`
+
 	// The number of writer goroutines processing the write channel.
 	WriteConcurrency int `toml:"write-concurrency"`
 
@@ -30,10 +41,12 @@ type Config struct {
 // NewConfig returns a new instance of a subscriber config.
 func NewConfig() Config {
 	return Config{
-		Enabled:          true,
-		HTTPTimeout:      toml.Duration(DefaultHTTPTimeout),
-		WriteConcurrency: DefaultWriteConcurrency,
-		WriteBufferSize:  DefaultWriteBufferSize,
+		Enabled:            true,
+		HTTPTimeout:        toml.Duration(DefaultHTTPTimeout),
+		InsecureSkipVerify: false,
+		CaCerts:            "",
+		WriteConcurrency:   DefaultWriteConcurrency,
+		WriteBufferSize:    DefaultWriteBufferSize,
 	}
 }
 
@@ -42,6 +55,14 @@ func (c Config) Validate() error {
 		return errors.New("http-timeout must be greater than 0")
 	}
 
+	if c.CaCerts != "" && !fileExists(c.CaCerts) {
+		abspath, err := filepath.Abs(c.CaCerts)
+		if err != nil {
+			return fmt.Errorf("ca-certs file %s does not exist. Wrapped Error: %v", c.CaCerts, err)
+		}
+		return fmt.Errorf("ca-certs file %s does not exist", abspath)
+	}
+
 	if c.WriteBufferSize <= 0 {
 		return errors.New("write-buffer-size must be greater than 0")
 	}
@@ -52,3 +73,8 @@ func (c Config) Validate() error {
 
 	return nil
 }
+
+func fileExists(fileName string) bool {
+	info, err := os.Stat(fileName)
+	return err == nil && !info.IsDir()
+}
