@@ -5,7 +5,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/influxdata/influxdb/cluster"
+	"github.com/influxdata/influxdb/coordinator"
 	"github.com/influxdata/influxdb/services/meta"
 	"github.com/influxdata/influxdb/services/subscriber"
 )
@@ -24,10 +24,10 @@ func (m MetaClient) WaitForDataChanged() chan struct{} {
 }
 
 type Subscription struct {
-	WritePointsFn func(*cluster.WritePointsRequest) error
+	WritePointsFn func(*coordinator.WritePointsRequest) error
 }
 
-func (s Subscription) WritePoints(p *cluster.WritePointsRequest) error {
+func (s Subscription) WritePoints(p *coordinator.WritePointsRequest) error {
 	return s.WritePointsFn(p)
 }
 
@@ -53,11 +53,11 @@ func TestService_IgnoreNonMatch(t *testing.T) {
 		}
 	}
 
-	prs := make(chan *cluster.WritePointsRequest, 2)
+	prs := make(chan *coordinator.WritePointsRequest, 2)
 	urls := make(chan url.URL, 2)
 	newPointsWriter := func(u url.URL) (subscriber.PointsWriter, error) {
 		sub := Subscription{}
-		sub.WritePointsFn = func(p *cluster.WritePointsRequest) error {
+		sub.WritePointsFn = func(p *coordinator.WritePointsRequest) error {
 			prs <- p
 			return nil
 		}
@@ -88,11 +88,11 @@ func TestService_IgnoreNonMatch(t *testing.T) {
 	}
 
 	// Write points that don't match any subscription.
-	s.Points() <- &cluster.WritePointsRequest{
+	s.Points() <- &coordinator.WritePointsRequest{
 		Database:        "db1",
 		RetentionPolicy: "rp0",
 	}
-	s.Points() <- &cluster.WritePointsRequest{
+	s.Points() <- &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp2",
 	}
@@ -128,11 +128,11 @@ func TestService_ModeALL(t *testing.T) {
 		}
 	}
 
-	prs := make(chan *cluster.WritePointsRequest, 2)
+	prs := make(chan *coordinator.WritePointsRequest, 2)
 	urls := make(chan url.URL, 2)
 	newPointsWriter := func(u url.URL) (subscriber.PointsWriter, error) {
 		sub := Subscription{}
-		sub.WritePointsFn = func(p *cluster.WritePointsRequest) error {
+		sub.WritePointsFn = func(p *coordinator.WritePointsRequest) error {
 			prs <- p
 			return nil
 		}
@@ -163,7 +163,7 @@ func TestService_ModeALL(t *testing.T) {
 	}
 
 	// Write points that match subscription with mode ALL
-	expPR := &cluster.WritePointsRequest{
+	expPR := &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp0",
 	}
@@ -171,7 +171,7 @@ func TestService_ModeALL(t *testing.T) {
 
 	// Should get pr back twice
 	for i := 0; i < 2; i++ {
-		var pr *cluster.WritePointsRequest
+		var pr *coordinator.WritePointsRequest
 		select {
 		case pr = <-prs:
 		case <-time.After(10 * time.Millisecond):
@@ -206,11 +206,11 @@ func TestService_ModeANY(t *testing.T) {
 		}
 	}
 
-	prs := make(chan *cluster.WritePointsRequest, 2)
+	prs := make(chan *coordinator.WritePointsRequest, 2)
 	urls := make(chan url.URL, 2)
 	newPointsWriter := func(u url.URL) (subscriber.PointsWriter, error) {
 		sub := Subscription{}
-		sub.WritePointsFn = func(p *cluster.WritePointsRequest) error {
+		sub.WritePointsFn = func(p *coordinator.WritePointsRequest) error {
 			prs <- p
 			return nil
 		}
@@ -240,14 +240,14 @@ func TestService_ModeANY(t *testing.T) {
 		}
 	}
 	// Write points that match subscription with mode ANY
-	expPR := &cluster.WritePointsRequest{
+	expPR := &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp0",
 	}
 	s.Points() <- expPR
 
 	// Validate we get the pr back just once
-	var pr *cluster.WritePointsRequest
+	var pr *coordinator.WritePointsRequest
 	select {
 	case pr = <-prs:
 	case <-time.After(10 * time.Millisecond):
@@ -294,11 +294,11 @@ func TestService_Multiple(t *testing.T) {
 		}
 	}
 
-	prs := make(chan *cluster.WritePointsRequest, 4)
+	prs := make(chan *coordinator.WritePointsRequest, 4)
 	urls := make(chan url.URL, 4)
 	newPointsWriter := func(u url.URL) (subscriber.PointsWriter, error) {
 		sub := Subscription{}
-		sub.WritePointsFn = func(p *cluster.WritePointsRequest) error {
+		sub.WritePointsFn = func(p *coordinator.WritePointsRequest) error {
 			prs <- p
 			return nil
 		}
@@ -329,24 +329,24 @@ func TestService_Multiple(t *testing.T) {
 	}
 
 	// Write points that don't match any subscription.
-	s.Points() <- &cluster.WritePointsRequest{
+	s.Points() <- &coordinator.WritePointsRequest{
 		Database:        "db1",
 		RetentionPolicy: "rp0",
 	}
-	s.Points() <- &cluster.WritePointsRequest{
+	s.Points() <- &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp2",
 	}
 
 	// Write points that match subscription with mode ANY
-	expPR := &cluster.WritePointsRequest{
+	expPR := &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp0",
 	}
 	s.Points() <- expPR
 
 	// Validate we get the pr back just once
-	var pr *cluster.WritePointsRequest
+	var pr *coordinator.WritePointsRequest
 	select {
 	case pr = <-prs:
 	case <-time.After(10 * time.Millisecond):
@@ -364,7 +364,7 @@ func TestService_Multiple(t *testing.T) {
 	}
 
 	// Write points that match subscription with mode ALL
-	expPR = &cluster.WritePointsRequest{
+	expPR = &coordinator.WritePointsRequest{
 		Database:        "db0",
 		RetentionPolicy: "rp1",
 	}
