@@ -7,7 +7,10 @@ import (
 	"math/rand"
 	"os"
 	"reflect"
+	"runtime"
+	"strings"
 	"sync"
+	"sync/atomic"
 	"testing"
 
 	"github.com/golang/snappy"
@@ -54,6 +57,27 @@ func TestCache_CacheWrite(t *testing.T) {
 	}
 }
 
+func TestCache_CacheWrite_TypeConflict(t *testing.T) {
+	v0 := NewValue(1, 1.0)
+	v1 := NewValue(2, int(64))
+	values := Values{v0, v1}
+	valuesSize := v0.Size() + v1.Size()
+
+	c := NewCache(uint64(2*valuesSize), "")
+
+	if err := c.Write("foo", values[:1]); err != nil {
+		t.Fatalf("failed to write key foo to cache: %s", err.Error())
+	}
+
+	if err := c.Write("foo", values[1:]); err == nil {
+		t.Fatalf("expected field type conflict")
+	}
+
+	if exp, got := uint64(v0.Size()), c.Size(); exp != got {
+		t.Fatalf("cache size incorrect after 2 writes, exp %d, got %d", exp, got)
+	}
+}
+
 func TestCache_CacheWriteMulti(t *testing.T) {
 	v0 := NewValue(1, 1.0)
 	v1 := NewValue(2, 2.0)
@@ -75,6 +99,28 @@ func TestCache_CacheWriteMulti(t *testing.T) {
 	}
 }
 
+func TestCache_CacheWriteMulti_TypeConflict(t *testing.T) {
+	v0 := NewValue(1, 1.0)
+	v1 := NewValue(2, 2.0)
+	v2 := NewValue(3, int64(3))
+	values := Values{v0, v1, v2}
+	valuesSize := uint64(v0.Size() + v1.Size() + v2.Size())
+
+	c := NewCache(3*valuesSize, "")
+
+	if err := c.WriteMulti(map[string][]Value{"foo": values[:1], "bar": values[1:]}); err == nil {
+		t.Fatalf(" expected field type conflict")
+	}
+
+	if exp, got := uint64(v0.Size()), c.Size(); exp != got {
+		t.Fatalf("cache size incorrect after 2 writes, exp %d, got %d", exp, got)
+	}
+
+	if exp, keys := []string{"foo"}, c.Keys(); !reflect.DeepEqual(keys, exp) {
+		t.Fatalf("cache keys incorrect after 2 writes, exp %v, got %v", exp, keys)
+	}
+}
+
 func TestCache_Cache_DeleteRange(t *testing.T) {
 	v0 := NewValue(1, 1.0)
 	v1 := NewValue(2, 2.0)
@@ -378,7 +424,7 @@ func TestCache_CacheWriteMemoryExceeded(t *testing.T) {
 	if exp, keys := []string{"foo"}, c.Keys(); !reflect.DeepEqual(keys, exp) {
 		t.Fatalf("cache keys incorrect after writes, exp %v, got %v", exp, keys)
 	}
-	if err := c.Write("bar", Values{v1}); err != ErrCacheMemoryExceeded {
+	if err := c.Write("bar", Values{v1}); err == nil || !strings.Contains(err.Error(), "cache-max-memory-size") {
 		t.Fatalf("wrong error writing key bar to cache")
 	}
 
@@ -387,7 +433,7 @@ func TestCache_CacheWriteMemoryExceeded(t *testing.T) {
 	if err != nil {
 		t.Fatalf("failed to snapshot cache: %v", err)
 	}
-	if err := c.Write("bar", Values{v1}); err != ErrCacheMemoryExceeded {
+	if err := c.Write("bar", Values{v1}); err == nil || !strings.Contains(err.Error(), "cache-max-memory-size") {
 		t.Fatalf("wrong error writing key bar to cache")
 	}
 
@@ -649,12 +695,49 @@ func mustMarshalEntry(entry WALEntry) (WalEntryType, []byte) {
 	return entry.Type(), snappy.Encode(b, b)
 }
 
+var fvSize = uint64(NewValue(1, float64(1)).Size())
+
 func BenchmarkCacheFloatEntries(b *testing.B) {
+	cache := NewCache(uint64(b.N)*fvSize, "")
+	vals := make([][]Value, b.N)
 	for i := 0; i < b.N; i++ {
-		cache := NewCache(10000, "")
-		for j := 0; j < 10000; j++ {
-			v := NewValue(1, float64(j))
-			cache.Write("test", []Value{v})
+		vals[i] = []Value{NewValue(1, float64(i))}
+	}
+	b.ResetTimer()
+
+	for i := 0; i < b.N; i++ {
+		if err := cache.Write("test", vals[i]); err != nil {
+			b.Fatal("err:", err, "i:", i, "N:", b.N)
 		}
 	}
 }
+
+type points struct {
+	key  string
+	vals []Value
+}
+
+func BenchmarkCacheParallelFloatEntries(b *testing.B) {
+	c := b.N * runtime.GOMAXPROCS(0)
+	cache := NewCache(uint64(c)*fvSize, "")
+	vals := make([]points, c)
+	for i := 0; i < c; i++ {
+		v := make([]Value, 10)
+		for j := 0; j < 10; j++ {
+			v[j] = NewValue(1, float64(i+j))
+		}
+		vals[i] = points{key: fmt.Sprintf("cpu%v", rand.Intn(20)), vals: v}
+	}
+	i := int32(-1)
+	b.ResetTimer()
+
+	b.RunParallel(func(pb *testing.PB) {
+		for pb.Next() {
+			j := atomic.AddInt32(&i, 1)
+			v := vals[j]
+			if err := cache.Write(v.key, v.vals); err != nil {
+				b.Fatal("err:", err, "j:", j, "N:", b.N)
+			}
+		}
+	})
+}
