@@ -1,6 +1,7 @@
 package tsm1
 
 import (
+	"bufio"
 	"encoding/binary"
 	"io/ioutil"
 	"math"
@@ -10,14 +11,23 @@ import (
 	"sync"
 )
 
-const v2header = 0x1502
+const (
+	v2header     = 0x1502
+	v2headerSize = 4
+)
 
 type Tombstoner struct {
-	mu sync.Mutex
+	mu sync.RWMutex
 
 	// Path is the location of the file to record tombstone. This should be the
 	// full path to a TSM file.
 	Path string
+
+	// cache of the stats for this tombstone
+	fileStats []FileStat
+	// indicates that the stats may be out of sync with what is on disk and they
+	// should be refreshed.
+	statsLoaded bool
 }
 
 type Tombstone struct {
@@ -49,6 +59,8 @@ func (t *Tombstoner) AddRange(keys []string, min, max int64) error {
 		return nil
 	}
 
+	t.statsLoaded = false
+
 	tombstones, err := t.readTombstone()
 	if err != nil {
 		return nil
@@ -75,34 +87,75 @@ func (t *Tombstoner) Delete() error {
 	if err := os.RemoveAll(t.tombstonePath()); err != nil {
 		return err
 	}
+	t.statsLoaded = false
 	return nil
 }
 
 // HasTombstones return true if there are any tombstone entries recorded.
 func (t *Tombstoner) HasTombstones() bool {
-	stat, err := os.Stat(t.tombstonePath())
-	if err != nil {
-		return false
-	}
-
-	return stat.Size() > 0
+	files := t.TombstoneFiles()
+	return len(files) > 0 && files[0].Size > 0
 }
 
 // TombstoneFiles returns any tombstone files associated with this TSM file.
 func (t *Tombstoner) TombstoneFiles() []FileStat {
+	t.mu.RLock()
+	if t.statsLoaded {
+		stats := t.fileStats
+		t.mu.RUnlock()
+		return stats
+	}
+	t.mu.RUnlock()
+
 	stat, err := os.Stat(t.tombstonePath())
-	if err != nil {
+	if os.IsNotExist(err) || err != nil {
+		t.mu.Lock()
+		// The file doesn't exist so record that we tried to load it so
+		// we don't continue to keep trying.  This is the common case.
+		t.statsLoaded = os.IsNotExist(err)
+		t.fileStats = t.fileStats[:0]
+		t.mu.Unlock()
 		return nil
 	}
 
-	if stat.Size() > 0 {
-		return []FileStat{FileStat{
-			Path:         stat.Name(),
-			LastModified: stat.ModTime().UnixNano(),
-			Size:         uint32(stat.Size())}}
+	t.mu.Lock()
+	t.fileStats = append(t.fileStats[:0], FileStat{
+		Path:         t.tombstonePath(),
+		LastModified: stat.ModTime().UnixNano(),
+		Size:         uint32(stat.Size()),
+	})
+	t.statsLoaded = true
+	stats := t.fileStats
+	t.mu.Unlock()
+
+	return stats
+}
+
+func (t *Tombstoner) Walk(fn func(t Tombstone) error) error {
+	f, err := os.Open(t.tombstonePath())
+	if os.IsNotExist(err) {
+		return nil
+	} else if err != nil {
+		return err
+	}
+	defer f.Close()
+
+	var b [4]byte
+	if _, err := f.Read(b[:]); err != nil {
+		// Might be a zero length file which should not exist, but
+		// an old bug allowed them to occur.  Treat it as an empty
+		// v1 tombstone file so we don't abort loading the TSM file.
+		return t.readTombstoneV1(f, fn)
 	}
 
-	return nil
+	if _, err := f.Seek(0, os.SEEK_SET); err != nil {
+		return err
+	}
+
+	if binary.BigEndian.Uint32(b[:]) == v2header {
+		return t.readTombstoneV2(f, fn)
+	}
+	return t.readTombstoneV1(f, fn)
 }
 
 func (t *Tombstoner) writeTombstone(tombstones []Tombstone) error {
@@ -154,78 +207,55 @@ func (t *Tombstoner) writeTombstone(tombstones []Tombstone) error {
 }
 
 func (t *Tombstoner) readTombstone() ([]Tombstone, error) {
-	f, err := os.Open(t.tombstonePath())
-	defer f.Close()
+	var tombstones []Tombstone
 
-	if !os.IsNotExist(err) {
-		var b [4]byte
-		_, err := f.Read(b[:])
-		if err != nil {
-			return nil, err
-		}
-
-		if _, err := f.Seek(0, os.SEEK_SET); err != nil {
-			return nil, err
-		}
-
-		if binary.BigEndian.Uint32(b[:]) == v2header {
-			return t.readTombstoneV2(f)
-		}
-
-		return t.readTombstoneV1(f)
+	if err := t.Walk(func(t Tombstone) error {
+		tombstones = append(tombstones, t)
+		return nil
+	}); err != nil {
+		return nil, err
 	}
-	return nil, nil
+	return tombstones, nil
 }
 
 // readTombstoneV1 reads the first version of tombstone files that were not
 // capable of storing a min and max time for a key.  This is used for backwards
 // compatibility with versions prior to 0.13.  This format is a simple newline
 // separated text file.
-func (t *Tombstoner) readTombstoneV1(f *os.File) ([]Tombstone, error) {
-	var b []byte
-	var err error
-	b, err = ioutil.ReadAll(f)
-	if err != nil {
-		return nil, err
-	}
-
-	lines := strings.TrimSpace(string(b))
-	if lines == "" {
-		return nil, nil
-	}
-
-	tombstones := []Tombstone{}
-
-	for _, line := range strings.Split(string(b), "\n") {
+func (t *Tombstoner) readTombstoneV1(f *os.File, fn func(t Tombstone) error) error {
+	r := bufio.NewScanner(f)
+	for r.Scan() {
+		line := r.Text()
 		if line == "" {
 			continue
 		}
-		tombstones = append(tombstones, Tombstone{
+		if err := fn(Tombstone{
 			Key: line,
 			Min: math.MinInt64,
 			Max: math.MaxInt64,
-		})
+		}); err != nil {
+			return err
+		}
 	}
-	return tombstones, nil
+	return r.Err()
 }
 
 // readTombstoneV2 reads the second version of tombstone files that are capable
 // of storing keys and the range of time for the key that points were deleted. This
 // format is binary.
-func (t *Tombstoner) readTombstoneV2(f *os.File) ([]Tombstone, error) {
+func (t *Tombstoner) readTombstoneV2(f *os.File, fn func(t Tombstone) error) error {
 	// Skip header, already checked earlier
-	if _, err := f.Seek(4, os.SEEK_SET); err != nil {
-		return nil, err
+	if _, err := f.Seek(v2headerSize, os.SEEK_SET); err != nil {
+		return err
 	}
 	n := int64(4)
 
 	fi, err := f.Stat()
 	if err != nil {
-		return nil, err
+		return err
 	}
 	size := fi.Size()
 
-	tombstones := []Tombstone{}
 	var (
 		min, max int64
 		key      string
@@ -233,11 +263,11 @@ func (t *Tombstoner) readTombstoneV2(f *os.File) ([]Tombstone, error) {
 	b := make([]byte, 4096)
 	for {
 		if n >= size {
-			return tombstones, nil
+			return nil
 		}
 
 		if _, err = f.Read(b[:4]); err != nil {
-			return nil, err
+			return err
 		}
 		n += 4
 
@@ -247,31 +277,32 @@ func (t *Tombstoner) readTombstoneV2(f *os.File) ([]Tombstone, error) {
 		}
 
 		if _, err := f.Read(b[:keyLen]); err != nil {
-			return nil, err
+			return err
 		}
 		key = string(b[:keyLen])
 		n += int64(keyLen)
 
 		if _, err := f.Read(b[:8]); err != nil {
-			return nil, err
+			return err
 		}
 		n += 8
 
 		min = int64(binary.BigEndian.Uint64(b[:8]))
 
 		if _, err := f.Read(b[:8]); err != nil {
-			return nil, err
+			return err
 		}
 		n += 8
 		max = int64(binary.BigEndian.Uint64(b[:8]))
 
-		tombstones = append(tombstones, Tombstone{
+		if err := fn(Tombstone{
 			Key: key,
 			Min: min,
 			Max: max,
-		})
+		}); err != nil {
+			return err
+		}
 	}
-	return tombstones, nil
 }
 
 func (t *Tombstoner) tombstonePath() string {
