@@ -3,10 +3,11 @@ package tsm1_test
 import (
 	"bytes"
 	"encoding/binary"
+	"io/ioutil"
+	"os"
 	"testing"
-	"time"
 
-	"github.com/influxdb/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
 )
 
 func TestTSMWriter_Write_Empty(t *testing.T) {
@@ -46,13 +47,16 @@ func TestTSMWriter_Write_NoValues(t *testing.T) {
 }
 
 func TestTSMWriter_Write_Single(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}
+	values := []tsm1.Value{tsm1.NewValue(0, 1.0)}
 	if err := w.Write("cpu", values); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 
@@ -65,17 +69,32 @@ func TestTSMWriter_Write_Single(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	if got, exp := len(b.Bytes()), 5; got < exp {
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	b, err := ioutil.ReadAll(fd)
+	if err != nil {
+		t.Fatalf("unexpected error reading: %v", err)
+	}
+
+	if got, exp := len(b), 5; got < exp {
 		t.Fatalf("file size mismatch: got %v, exp %v", got, exp)
 	}
-	if got := binary.BigEndian.Uint32(b.Bytes()[0:4]); got != tsm1.MagicNumber {
+	if got := binary.BigEndian.Uint32(b[0:4]); got != tsm1.MagicNumber {
 		t.Fatalf("magic number mismatch: got %v, exp %v", got, tsm1.MagicNumber)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	if _, err := fd.Seek(0, os.SEEK_SET); err != nil {
+		t.Fatalf("unexpected error seeking: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	readValues, err := r.ReadAll("cpu")
 	if err != nil {
@@ -94,8 +113,11 @@ func TestTSMWriter_Write_Single(t *testing.T) {
 }
 
 func TestTSMWriter_Write_Multiple(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -104,8 +126,8 @@ func TestTSMWriter_Write_Multiple(t *testing.T) {
 		key    string
 		values []tsm1.Value
 	}{
-		{"cpu", []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}},
-		{"mem", []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), 2.0)}},
+		{"cpu", []tsm1.Value{tsm1.NewValue(0, 1.0)}},
+		{"mem", []tsm1.Value{tsm1.NewValue(1, 2.0)}},
 	}
 
 	for _, d := range data {
@@ -122,10 +144,16 @@ func TestTSMWriter_Write_Multiple(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	for _, d := range data {
 		readValues, err := r.ReadAll(d.key)
@@ -146,8 +174,11 @@ func TestTSMWriter_Write_Multiple(t *testing.T) {
 }
 
 func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -157,12 +188,12 @@ func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.0),
-			tsm1.NewValue(time.Unix(1, 0), 2.0)},
+			tsm1.NewValue(0, 1.0),
+			tsm1.NewValue(1, 2.0)},
 		},
 		{"mem", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.5),
-			tsm1.NewValue(time.Unix(1, 0), 2.5)},
+			tsm1.NewValue(0, 1.5),
+			tsm1.NewValue(1, 2.5)},
 		},
 	}
 
@@ -180,10 +211,16 @@ func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	for _, d := range data {
 		readValues, err := r.ReadAll(d.key)
@@ -205,8 +242,11 @@ func TestTSMWriter_Write_MultipleKeyValues(t *testing.T) {
 
 // Tests that writing keys in reverse is able to read them back.
 func TestTSMWriter_Write_ReverseKeys(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -216,12 +256,12 @@ func TestTSMWriter_Write_ReverseKeys(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"mem", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.5),
-			tsm1.NewValue(time.Unix(1, 0), 2.5)},
+			tsm1.NewValue(0, 1.5),
+			tsm1.NewValue(1, 2.5)},
 		},
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.0),
-			tsm1.NewValue(time.Unix(1, 0), 2.0)},
+			tsm1.NewValue(0, 1.0),
+			tsm1.NewValue(1, 2.0)},
 		},
 	}
 
@@ -239,10 +279,16 @@ func TestTSMWriter_Write_ReverseKeys(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	for _, d := range data {
 		readValues, err := r.ReadAll(d.key)
@@ -264,8 +310,11 @@ func TestTSMWriter_Write_ReverseKeys(t *testing.T) {
 
 // Tests that writing keys in reverse is able to read them back.
 func TestTSMWriter_Write_SameKey(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -275,12 +324,12 @@ func TestTSMWriter_Write_SameKey(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.0),
-			tsm1.NewValue(time.Unix(1, 0), 2.0)},
+			tsm1.NewValue(0, 1.0),
+			tsm1.NewValue(1, 2.0)},
 		},
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(2, 0), 3.0),
-			tsm1.NewValue(time.Unix(3, 0), 4.0)},
+			tsm1.NewValue(2, 3.0),
+			tsm1.NewValue(3, 4.0)},
 		},
 	}
 
@@ -298,10 +347,16 @@ func TestTSMWriter_Write_SameKey(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	values := append(data[0].values, data[1].values...)
 
@@ -324,8 +379,11 @@ func TestTSMWriter_Write_SameKey(t *testing.T) {
 // Tests that calling Read returns all the values for block matching the key
 // and timestamp
 func TestTSMWriter_Read_Multiple(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -335,12 +393,12 @@ func TestTSMWriter_Read_Multiple(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(0, 0), 1.0),
-			tsm1.NewValue(time.Unix(1, 0), 2.0)},
+			tsm1.NewValue(0, 1.0),
+			tsm1.NewValue(1, 2.0)},
 		},
 		{"cpu", []tsm1.Value{
-			tsm1.NewValue(time.Unix(2, 0), 3.0),
-			tsm1.NewValue(time.Unix(3, 0), 4.0)},
+			tsm1.NewValue(2, 3.0),
+			tsm1.NewValue(3, 4.0)},
 		},
 	}
 
@@ -358,14 +416,20 @@ func TestTSMWriter_Read_Multiple(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	for _, values := range data {
 		// Try the first timestamp
-		readValues, err := r.Read("cpu", values.values[0].Time())
+		readValues, err := r.Read("cpu", values.values[0].UnixNano())
 		if err != nil {
 			t.Fatalf("unexpected error readin: %v", err)
 		}
@@ -381,7 +445,7 @@ func TestTSMWriter_Read_Multiple(t *testing.T) {
 		}
 
 		// Try the last timestamp too
-		readValues, err = r.Read("cpu", values.values[1].Time())
+		readValues, err = r.Read("cpu", values.values[1].UnixNano())
 		if err != nil {
 			t.Fatalf("unexpected error readin: %v", err)
 		}
@@ -399,14 +463,16 @@ func TestTSMWriter_Read_Multiple(t *testing.T) {
 }
 
 func TestTSMWriter_WriteBlock_Empty(t *testing.T) {
-	// Write a new TSM file
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	if err := w.WriteBlock("cpu", time.Unix(0, 0), time.Unix(0, 0), nil); err != nil {
+	if err := w.WriteBlock("cpu", 0, 0, nil); err != nil {
 		t.Fatalf("unexpected error writing block: %v", err)
 	}
 
@@ -414,15 +480,28 @@ func TestTSMWriter_WriteBlock_Empty(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	if got, exp := len(b.Bytes()), 0; got < exp {
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+	defer fd.Close()
+
+	b, err := ioutil.ReadAll(fd)
+	if err != nil {
+		t.Fatalf("unexpected error read all: %v", err)
+	}
+
+	if got, exp := len(b), 0; got < exp {
 		t.Fatalf("file size mismatch: got %v, exp %v", got, exp)
 	}
 }
 
 func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
-	// Write a new TSM file
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
@@ -431,8 +510,8 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 		key    string
 		values []tsm1.Value
 	}{
-		{"cpu", []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}},
-		{"mem", []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), 2.0)}},
+		{"cpu", []tsm1.Value{tsm1.NewValue(0, 1.0)}},
+		{"mem", []tsm1.Value{tsm1.NewValue(1, 2.0)}},
 	}
 
 	for _, d := range data {
@@ -449,29 +528,43 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	if got, exp := len(b.Bytes()), 5; got < exp {
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+	defer fd.Close()
+
+	b, err := ioutil.ReadAll(fd)
+	if err != nil {
+		t.Fatalf("unexpected error read all: %v", err)
+	}
+
+	if got, exp := len(b), 5; got < exp {
 		t.Fatalf("file size mismatch: got %v, exp %v", got, exp)
 	}
-	if got := binary.BigEndian.Uint32(b.Bytes()[0:4]); got != tsm1.MagicNumber {
+	if got := binary.BigEndian.Uint32(b[0:4]); got != tsm1.MagicNumber {
 		t.Fatalf("magic number mismatch: got %v, exp %v", got, tsm1.MagicNumber)
 	}
 
+	if _, err := fd.Seek(0, os.SEEK_SET); err != nil {
+		t.Fatalf("error seeking: %v", err)
+	}
+
 	// Create reader for that file
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
 
-	// Using the reader, write a new file using WriteBlocks
-	b.Reset()
-	w, err = tsm1.NewTSMWriter(&b)
+	f = MustTempFile(dir)
+	w, err = tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
 	iter := r.BlockIterator()
 	for iter.Next() {
-		key, minTime, maxTime, b, err := iter.Read()
+		key, minTime, maxTime, _, b, err := iter.Read()
 		if err != nil {
 			t.Fatalf("unexpected error reading block: %v", err)
 		}
@@ -487,12 +580,18 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
+	fd, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
 	// Now create a reader to verify the written blocks matches the originally
 	// written file using Write
-	r, err = tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	r, err = tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
+	defer r.Close()
 
 	for _, d := range data {
 		readValues, err := r.ReadAll(d.key)
@@ -511,3 +610,23 @@ func TestTSMWriter_WriteBlock_Multiple(t *testing.T) {
 		}
 	}
 }
+
+func TestTSMWriter_Write_MaxKey(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error created writer: %v", err)
+	}
+
+	var key string
+	for i := 0; i < 100000; i++ {
+		key += "a"
+	}
+	if err := w.Write(key, []tsm1.Value{tsm1.NewValue(0, 1.0)}); err != tsm1.ErrMaxKeyLengthExceeded {
+		t.Fatalf("expected max key length error writing key: %v", err)
+	}
+}
